@@ -216,6 +216,7 @@ package Sidef::Optimizer {
                    round roundf
                    digit
                    nok
+                   modinv
                    rdiv
                    is_div
 
@@ -720,28 +721,26 @@ package Sidef::Optimizer {
             }
         }
 
-        if (not exists($expr->{ind}) and not exists($expr->{lookup}) and not exists($expr->{call})) {
+        if (not exists($expr->{ind}) and not exists($expr->{call})) {
             return (ref($obj) eq 'HASH' ? {self => $obj} : $obj);
         }
 
         $obj = {
                 self => $obj,
-                (exists($expr->{ind})    ? (ind    => []) : ()),
-                (exists($expr->{lookup}) ? (lookup => []) : ()),
-                (exists($expr->{call})   ? (call   => []) : ()),
+                (exists($expr->{ind})  ? (ind  => []) : ()),
+                (exists($expr->{call}) ? (call => []) : ()),
                };
 
-        # Array indices
+        # Array and hash indices
         if (exists $expr->{ind}) {
             foreach my $i (0 .. $#{$expr->{ind}}) {
-                $obj->{ind}[$i] = [map { $self->optimize_expr($_) } @{$expr->{ind}[$i]}];
-            }
-        }
-
-        # Hash lookup
-        if (exists $expr->{lookup}) {
-            foreach my $i (0 .. $#{$expr->{lookup}}) {
-                $obj->{lookup}[$i] = [map { ref($_) eq 'HASH' ? $self->optimize_expr($_) : $_ } @{$expr->{lookup}[$i]}];
+                my $ind = $expr->{ind}[$i];
+                if (exists $ind->{array}) {
+                    $obj->{ind}[$i]{array} = [map { ref($_) eq 'HASH' ? $self->optimize_expr($_) : $_ } @{$ind->{array}}];
+                }
+                else {
+                    $obj->{ind}[$i]{hash} = [map { ref($_) eq 'HASH' ? $self->optimize_expr($_) : $_ } @{$ind->{hash}}];
+                }
             }
         }
 
@@ -793,7 +792,6 @@ package Sidef::Optimizer {
                 if (    defined($ref_obj)
                     and exists($rules{$ref_obj})
                     and not exists($expr->{ind})
-                    and not exists($expr->{lookup})
                     and ref($method) eq '') {
 
                     my $code = $ref_obj->SUPER::can($method);
@@ -849,7 +847,7 @@ package Sidef::Optimizer {
 
             if ($count > 0) {
                 if ($count == @{$obj->{call}}) {
-                    if (not exists $expr->{ind} and not exists $expr->{lookup}) {
+                    if (not exists $expr->{ind}) {
                         return $obj->{self};
                     }
                     else {
