@@ -1,7 +1,7 @@
 package Sidef::Types::Array::Array {
 
     use utf8;
-    use 5.014;
+    use 5.016;
 
     use parent qw(
       Sidef::Object::Object
@@ -88,10 +88,10 @@ package Sidef::Types::Array::Array {
         my ($self, $operator) = @_;
 
         $operator = $operator->get_value if ref($operator);
-        (my $offset = $#{$self}) >= 0 || return;
+        (my $end = $#{$self}) >= 0 || return;
 
         my $x = $self->[0];
-        foreach my $i (1 .. $offset) {
+        foreach my $i (1 .. $end) {
             $x = $x->$operator($self->[$i]);
         }
         $x;
@@ -166,7 +166,13 @@ package Sidef::Types::Array::Array {
 
     sub multiply {
         my ($self, $num) = @_;
-        $self->new((@{$self}) x $num->get_value);
+
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $num = $num->get_value;
+        }
+
+        $self->new((@{$self}) x $num);
     }
 
     *mul = \&multiply;
@@ -177,7 +183,10 @@ package Sidef::Types::Array::Array {
         my @obj = @{$self};
 
         my @array;
-        my $len = @obj / $num->get_value;
+        my $len = @obj / do {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $num->get_value;
+        };
 
         my $i   = 1;
         my $pos = $len;
@@ -197,14 +206,12 @@ package Sidef::Types::Array::Array {
     sub or {
         my ($self, $array) = @_;
         my $new_array = $self->new;
-
         $self->xor($array)->concat($self->and($array));
     }
 
     sub xor {
         my ($self, $array) = @_;
         my $new_array = $self->new;
-
         ($self->concat($array))->subtract($self->and($array));
     }
 
@@ -260,23 +267,67 @@ package Sidef::Types::Array::Array {
     *lev   = \&levenshtein;
     *leven = \&levenshtein;
 
-    sub _combinations {
-        my ($n, @set) = @_;
-
-        @set || return;
-        $n == 1 && return map { [$_] } @set;
-
-        my $head = shift @set;
-        my @result = _combinations($n - 1, @set);
-        foreach my $subarray (@result) {
-            CORE::unshift @{$subarray}, $head;
-        }
-        @result, _combinations($n, @set);
-    }
-
     sub combinations {
-        my ($self, $n) = @_;
-        Sidef::Types::Array::Array->new(map { Sidef::Types::Array::Array->new(@{$_}) } _combinations($n->get_value, @{$self}));
+        my ($self, $k, $block) = @_;
+
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $k = $k->get_value;
+        }
+
+        if (defined($block)) {
+
+            if ($k == 0) {
+                if (defined(my $res = $block->_run_code($self->new))) {
+                    return $res;
+                }
+                return $self;
+            }
+
+            return if $k < 0;
+
+            my $n = @{$self};
+            return $self if ($k > $n or $n == 0);
+
+            my @c = (0 .. $k - 1);
+
+            while (1) {
+
+                if (defined(my $res = $block->_run_code($self->new(@{$self}[@c])))) {
+                    return $res;
+                }
+
+                next if ($c[$k - 1]++ < $n - 1);
+                my $i = $k - 2;
+                $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
+                last if $i < 0;
+                $c[$i]++;
+                while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
+            }
+
+            return $self;
+        }
+
+        return $self->new($self->new) if $k == 0;
+        return if $k < 0;
+
+        my $n = @{$self};
+        return $self->new if ($k > $n or $n == 0);
+
+        my @c = (0 .. $k - 1);
+        my @result;
+
+        while (1) {
+            push @result, $self->new(@{$self}[@c]);
+            next if ($c[$k - 1]++ < $n - 1);
+            my $i = $k - 2;
+            $i-- while ($i >= 0 && $c[$i] >= $n - ($k - $i));
+            last if $i < 0;
+            $c[$i]++;
+            while (++$i < $k) { $c[$i] = $c[$i - 1] + 1; }
+        }
+
+        $self->new(@result);
     }
 
     *combination = \&combinations;
@@ -358,7 +409,12 @@ package Sidef::Types::Array::Array {
 
     sub make {
         my ($self, $size, $type) = @_;
-        $self->new(($type) x $size->get_value);
+        $self->new(
+            ($type) x do {
+                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                $size->get_value;
+              }
+        );
     }
 
     sub _min_max {
@@ -481,7 +537,10 @@ package Sidef::Types::Array::Array {
                 return $self->last_by($arg);
             }
 
-            my $from = @{$self} - $arg->get_value;
+            my $from = @{$self} - do {
+                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                $arg->get_value;
+            };
             return $self->new(@{$self}[($from < 0 ? 0 : $from) .. $#{$self}]);
         }
 
@@ -512,6 +571,8 @@ package Sidef::Types::Array::Array {
         $self->new(@new_array);
     }
 
+    *flat = \&flatten;
+
     sub exists {
         my ($self, $index) = @_;
         Sidef::Types::Bool::Bool->new(exists $self->[$index->get_value]);
@@ -556,8 +617,11 @@ package Sidef::Types::Array::Array {
 
         my $max = @{$self};
 
-        $from = defined($from) ? ($from->get_value) : 0;
-        $to   = defined($to)   ? ($to->get_value)   : $max - 1;
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $from = defined($from) ? ($from->get_value) : 0;
+            $to   = defined($to)   ? ($to->get_value)   : $max - 1;
+        }
 
         if (abs($from) > $max) {
             return;
@@ -608,6 +672,48 @@ package Sidef::Types::Array::Array {
     *for     = \&each;
     *foreach = \&each;
 
+    sub each_slice {
+        my ($self, $n, $code) = @_;
+
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $n = $n->get_value;
+        }
+
+        my $end = @{$self};
+        for (my $i = $n - 1 ; $i < $end ; $i += $n) {
+            if (defined(my $res = $code->_run_code($self->new(@{$self}[$i - ($n - 1) .. $i])))) {
+                return $res;
+            }
+        }
+
+        my $mod = $end % $n;
+        if ($mod != 0) {
+            if (defined(my $res = $code->_run_code($self->new(@{$self}[$end - $mod .. $end - 1])))) {
+                return $res;
+            }
+        }
+
+        $self;
+    }
+
+    sub each_cons {
+        my ($self, $n, $code) = @_;
+
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $n = $n->get_value;
+        }
+
+        foreach my $i ($n - 1 .. $#{$self}) {
+            if (defined(my $res = $code->_run_code($self->new(@{$self}[$i - $n + 1 .. $i])))) {
+                return $res;
+            }
+        }
+
+        $self;
+    }
+
     sub each_index {
         my ($self, $code) = @_;
 
@@ -620,7 +726,9 @@ package Sidef::Types::Array::Array {
         $self;
     }
 
-    sub each_with_index {
+    *each_key = \&each_index;
+
+    sub each_kv {
         my ($self, $code) = @_;
 
         foreach my $i (0 .. $#{$self}) {
@@ -639,7 +747,7 @@ package Sidef::Types::Array::Array {
 
     *collect = \&map;
 
-    sub map_with_index {
+    sub map_kv {
         my ($self, $code) = @_;
 
         my @arr;
@@ -650,7 +758,7 @@ package Sidef::Types::Array::Array {
         $self->new(@arr);
     }
 
-    *collect_with_index = \&map_with_index;
+    *collect_kv = \&map_kv;
 
     sub flat_map {
         my ($self, $code) = @_;
@@ -665,6 +773,20 @@ package Sidef::Types::Array::Array {
     *filter = \&grep;
     *select = \&grep;
 
+    sub grep_kv {
+        my ($self, $code) = @_;
+
+        my @arr;
+        foreach my $i (0 .. $#{$self}) {
+            push(@arr, $self->[$i]) if $code->run(Sidef::Types::Number::Number->new($i), $self->[$i]);
+        }
+
+        $self->new(@arr);
+    }
+
+    *filter_kv = \&grep_kv;
+    *select_kv = \&grep_kv;
+
     sub group_by {
         my ($self, $code) = @_;
 
@@ -806,24 +928,14 @@ package Sidef::Types::Array::Array {
 
     *last_index = \&rindex;
 
-    sub reduce_pairs {
+    sub pairmap {
         my ($self, $obj) = @_;
 
-        (my $end = $#{$self}) == -1
-          && return $self->new;
+        my $end = @{$self} || return $self->new;
 
         my @array;
-        if (ref($obj) eq 'Sidef::Types::Block::Block') {
-            for (my $i = 1 ; $i <= $end ; $i += 2) {
-                push @array, scalar $obj->run($self->[$i - 1], $self->[$i]);
-            }
-        }
-        else {
-            my $method = $obj->get_value;
-            for (my $i = 1 ; $i <= $end ; $i += 2) {
-                my $x = $self->[$i - 1];
-                push @array, $x->$method($self->[$i]);
-            }
+        for (my $i = 1 ; $i < $end ; $i += 2) {
+            push @array, scalar $obj->run(@{$self}[$i - 1, $i]);
         }
 
         $self->new(@array);
@@ -897,7 +1009,8 @@ package Sidef::Types::Array::Array {
 
     sub resize {
         my ($self, $num) = @_;
-        $#{$self} = $num;
+        local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+        $#{$self} = $num->get_value;
         $num;
     }
 
@@ -905,7 +1018,12 @@ package Sidef::Types::Array::Array {
 
     sub rand {
         my ($self, $amount) = @_;
+
         if (defined $amount) {
+            {
+                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                $amount = $amount->get_value;
+            }
             return $self->new(map { $self->[CORE::rand(scalar @{$self})] } 1 .. $amount);
         }
         $self->[CORE::rand(scalar @{$self})];
@@ -927,7 +1045,7 @@ package Sidef::Types::Array::Array {
 
     sub insert {
         my ($self, $index, @objects) = @_;
-        splice(@{$self}, $index->get_value, 0, @{__PACKAGE__->new(@objects)});
+        splice(@{$self}, $index->get_value, 0, @objects);
         $self;
     }
 
@@ -1031,59 +1149,52 @@ package Sidef::Types::Array::Array {
     *last_unique_by = \&last_uniq_by;
 
     sub abbrev {
-        my ($self, $code) = @_;
+        my ($self, $block) = @_;
 
-        my $__END__ = {};                                                                # some unique value
-        my $__CALL__ = defined($code) && (ref($code) eq 'Sidef::Types::Block::Block');
+        my $tail     = {};                # some unique value
+        my $callback = defined($block);
 
         my %table;
         foreach my $sub_array (@{$self}) {
             my $ref = \%table;
-
             foreach my $item (@{$sub_array}) {
                 $ref = $ref->{$item} //= {};
             }
-            $ref->{$__END__} = $sub_array;
+            $ref->{$tail} = $sub_array;
         }
 
-        my $abbrevs = $__CALL__ ? undef : $self->new();
-        my $callback = sub {
-            $abbrevs->append($self->new(@_));
-        };
+        my @abbrev;
+        sub {
+            my ($hash) = @_;
 
-        my $traverse;
-        (
-         $traverse = sub {
-             my ($hash) = @_;
+            foreach my $key (my @keys = CORE::sort keys %{$hash}) {
+                next if $key eq $tail;
+                __SUB__->($hash->{$key});
 
-             foreach my $key (my @keys = CORE::sort keys %{$hash}) {
-                 next if $key eq $__END__;
-                 $traverse->($hash->{$key});
+                if ($#keys > 0) {
+                    my $count = 0;
+                    my $ref   = delete $hash->{$key};
+                    while (my ($key) = CORE::each %{$ref}) {
+                        if ($key eq $tail) {
 
-                 if ($#keys > 0) {
-                     my $count = 0;
-                     my $ref   = delete $hash->{$key};
-                     while (my ($key) = CORE::each %{$ref}) {
-                         if ($key eq $__END__) {
+                            if ($callback) {
+                                $block->run($self->new(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]));
+                            }
+                            else {
+                                push @abbrev, $self->new(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]);
+                            }
 
-                             if ($__CALL__) {
-                                 $code->run($self->new(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]));
-                             }
-                             else {
-                                 $callback->(@{$ref->{$key}}[0 .. $#{$ref->{$key}} - $count]);
-                             }
+                            last;
+                        }
+                        $ref = $ref->{$key};
+                        $count++;
+                    }
+                }
+            }
+          }
+          ->(\%table);
 
-                             last;
-                         }
-                         $ref = $ref->{$key};
-                         $count++;
-                     }
-                 }
-             }
-         }
-        )->(\%table);
-
-        $abbrevs;
+        $self->new(@abbrev);
     }
 
     *abbreviations = \&abbrev;
@@ -1152,7 +1263,16 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         if (defined $num) {
-            return $self->new(CORE::splice(@{$self}, 0, $num->get_value));
+            return $self->new(
+                CORE::splice(
+                    @{$self},
+                    0,
+                    do {
+                        local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                        $num->get_value;
+                      }
+                )
+            );
         }
 
         @{$self} || return;
@@ -1166,7 +1286,12 @@ package Sidef::Types::Array::Array {
         my ($self, $num) = @_;
 
         if (defined $num) {
-            $num = $num->get_value > $#{$self} ? 0 : @{$self} - $num->get_value;
+            {
+                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                $num = $num->get_value;
+            }
+
+            $num = $num > $#{$self} ? 0 : @{$self} - $num;
             return $self->new(CORE::splice(@{$self}, $num));
         }
 
@@ -1185,7 +1310,14 @@ package Sidef::Types::Array::Array {
 
     sub delete_index {
         my ($self, $offset) = @_;
-        CORE::splice(@{$self}, $offset->get_value, 1);
+        CORE::splice(
+            @{$self},
+            do {
+                local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+                $offset->get_value;
+            },
+            1
+                    );
     }
 
     *pop_at    = \&delete_index;
@@ -1194,28 +1326,36 @@ package Sidef::Types::Array::Array {
     sub splice {
         my ($self, $offset, $length, @objects) = @_;
 
-        $offset = defined($offset) ? $offset->get_value : 0;
-        $length = defined($length) ? $length->get_value : scalar(@{$self});
-
-        if (@objects) {
-            return $self->new(CORE::splice(@{$self}, $offset, $length, @{__PACKAGE__->new(@objects)}));
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $offset = defined($offset) ? $offset->get_value : 0;
+            $length = defined($length) ? $length->get_value : scalar(@{$self});
         }
 
-        $self->new(CORE::splice(@{$self}, $offset, $length));
+        $self->new(CORE::splice(@{$self}, $offset, $length, @objects));
     }
 
     sub take_right {
         my ($self, $amount) = @_;
 
-        my $offset = $#{$self};
-        $amount = $offset > ($amount->get_value - 1) ? $amount->get_value - 1 : $offset;
-        $self->new(@{$self}[$offset - $amount .. $offset]);
+        my $end = $#{$self};
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $amount = $amount->get_value;
+            $amount = $end > ($amount - 1) ? $amount - 1 : $end;
+        }
+        $self->new(@{$self}[$end - $amount .. $end]);
     }
 
     sub take_left {
         my ($self, $amount) = @_;
 
-        $amount = $#{$self} > ($amount->get_value - 1) ? $amount->get_value - 1 : $#{$self};
+        my $end = $#{$self};
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $amount = $amount->get_value;
+            $amount = $end > ($amount - 1) ? $amount - 1 : $end;
+        }
         $self->new(@{$self}[0 .. $amount]);
     }
 
@@ -1332,7 +1472,12 @@ package Sidef::Types::Array::Array {
     sub rotate {
         my ($self, $num) = @_;
 
-        $num = $num->get_value % ($#{$self} + 1);
+        {
+            local $Sidef::Types::Number::Number::GET_PERL_VALUE = 1;
+            $num = $num->get_value;
+        }
+
+        $num %= ($#{$self} + 1);
         return $self->new(@{$self}) if $num == 0;
 
         # Surprisingly, this is slower:
@@ -1469,9 +1614,8 @@ package Sidef::Types::Array::Array {
 
     *remove_last_if = \&delete_last_if;
 
-    sub to_list {
-        @{$_[0]};
-    }
+    sub to_list { @{$_[0]} }
+    *as_list = \&to_list;
 
     sub dump {
         my ($self) = @_;
