@@ -33,13 +33,12 @@ package Sidef::Types::Array::Pair {
         \@array;
     }
 
-    {
-        no strict 'refs';
-        foreach my $pair ([first => 0], [second => 1]) {
-            *{__PACKAGE__ . '::' . $pair->[0]} = sub {
-                $_[0]->[$pair->[1]];
-            };
-        }
+    sub first : lvalue {
+        $_[0][0];
+    }
+
+    sub second : lvalue {
+        $_[0][1];
     }
 
     sub swap {
