@@ -7,7 +7,6 @@ package Sidef::Types::Byte::Byte {
 
     sub new {
         my (undef, $byte) = @_;
-        state $x = require Math::BigInt;
         bless \Math::BigInt->new($byte), __PACKAGE__;
     }
 
@@ -15,7 +14,7 @@ package Sidef::Types::Byte::Byte {
 
     sub dump {
         my ($self) = @_;
-        Sidef::Types::String::String->new('Byte.new(' . $self->get_value . ')');
+        Sidef::Types::String::String->new('Byte(' . $self->get_value . ')');
     }
 };
 
