@@ -28,11 +28,15 @@ use Sidef;
 
 use File::Find qw(find);
 use File::Basename qw(basename);
+use File::Spec::Functions qw(catdir updir rel2abs);
+
+local $ENV{SIDEF_INC} = rel2abs(catdir(updir(), 'scripts', 'Tests'));
 
 sub parse_deparse {
     my ($code, $name) = @_;
 
-    local @Sidef::Exec::NAMESPACES = ();
+    local %Sidef::INCLUDED   = ();
+    local @Sidef::NAMESPACES = ();
 
     my $parser = Sidef::Parser->new(
                                     file_name   => $name,
@@ -42,16 +46,14 @@ sub parse_deparse {
 
     my $struct = $parser->parse_script(code => \$code);
 
-    my $deparser   = Sidef::Deparse::Sidef->new(namespaces => [@Sidef::Exec::NAMESPACES]);
+    my $deparser   = Sidef::Deparse::Sidef->new(namespaces => [@Sidef::NAMESPACES]);
     my @statements = $deparser->deparse_script($struct);
     my $deparsed   = $deparser->{before} . join($deparser->{between}, @statements) . $deparser->{after};
 
     return ($deparsed, \@statements);
 }
 
-my %ignore = (
-    'Matrix.sm'                         => 1,
-);
+my %ignore = ('module_definition.sf' => 1,);
 
 my $dir = shift() // die "usage: $0 [scripts dir]\n";
 
@@ -66,20 +68,6 @@ sub test_file {
     my $basename = basename($file);
     return if exists $ignore{$basename};
 
-    delete @INC{
-        qw(
-          Math/BigInt.pm
-          Math/BigFloat.pm
-          Math/BigRat.pm
-          Sidef/Types/Number/Number.pm
-          Sidef/Types/Number/NumberInt.pm
-          Sidef/Types/Number/NumberFast.pm
-          Sidef/Types/Number/NumberRat.pm
-          )
-    };
-
-    require Sidef::Types::Number::Number;
-
     {
         local $| = 1;
         printf("** Processing: %s\r", $file);
